#lang racket

; write-numlist : list-of-num output-port -> void
;   Effect: write l to p
(define (write-numlist l p)
  (cond
    [(empty? l) (write-char #\. p)]
    [else (begin
            (write-num (first l) p)
            (write-char #\space p)
            (write-numlist (rest l) p))]))

; write-num : num output-port -> void
;   Effect: write n to p
(define (write-num n p)
  (cond
    [(< n 10) (write-digit n p)]
    [else (begin
            (write-num (quotient n 10) p)
            (write-digit (remainder n 10) p))]))

; write-digit : num(0-9) output-port -> void
;   Effect: write n to p
(define (write-digit n p)
  (cond
    ((= n 0) (write-char #\0 p))
    ((= n 1) (write-char #\1 p))
    ((= n 2) (write-char #\2 p))
    ((= n 3) (write-char #\3 p))
    ((= n 4) (write-char #\4 p))
    ((= n 5) (write-char #\5 p))
    ((= n 6) (write-char #\6 p))
    ((= n 7) (write-char #\7 p))
    ((= n 8) (write-char #\8 p))
    ((= n 9) (write-char #\9 p))))


; read-numlist : input-port -> list-of-num
(define (read-numlist p)
  (local ((define c (read-char p)))
    (cond
      ((char=? #\. c) empty)
      ((char-digit? c) (cons (read-number p (digit-val c))
                             (read-numlist p))))))

; read-number : input-port num -> num
(define (read-number p n)
  (local ((define c (read-char p)))
    (cond
      ((char=? #\space c) n)
      ((char-digit? c)
       (read-number p (+ (* n 10) (digit-val c)))))))

; char-digit? : char -> bool
(define (char-digit? c)
  (or (char=? c #\0) (char=? c #\1) (char=? c #\2)
      (char=? c #\3) (char=? c #\4) (char=? c #\5)
      (char=? c #\6) (char=? c #\7) (char=? c #\8)
      (char=? c #\9)))

; digit-val : char -> num
(define (digit-val c)
  (cond
    ((char=? c #\0) 0)
    ((char=? c #\1) 1)
    ((char=? c #\2) 2)
    ((char=? c #\3) 3)
    ((char=? c #\4) 4)
    ((char=? c #\5) 5)
    ((char=? c #\6) 6)
    ((char=? c #\7) 7)
    ((char=? c #\8) 8)
    ((char=? c #\9) 9)))
